<?php

namespace FiloBlu\Refilo\Model\Indexer\Processor\Type;

use Magento\Catalog\Model\Product\Attribute\Source\Status;
use Magento\Catalog\Model\ProductRepository;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Api\Search\FilterGroup;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Framework\Api\SearchCriteriaInterface;

class Bundle extends Simple
{
    /**
     * @var ProductRepository
     */
    private $productRepository;
    /**
     * @var FilterGroup
     */
    private $filterGroup;
    /**
     * @var FilterBuilder
     */
    private $filterBuilder;
    /**
     * @var Status
     */
    private $productStatus;
    /**
     * @var Simple
     */
    private $_simple_product;

    /**
     * @var array
     */
    private $child_products;
    /**
     * @var SearchCriteriaInterface
     */
    private $criteria;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;

    /**
     * @var array
     */
    private $bundle_options;

    /**
     * @var string
     */
    public $price_type;

    /**
     * Configurable constructor.
     * @param ProductRepository $productRepository
     * @param FilterGroup $filterGroup
     * @param FilterBuilder $filterBuilder
     * @param Status $productStatus
     * @param Simple $simple_product
     * @param SearchCriteriaInterface $criteria
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     */
    public function __construct(
        ProductRepository $productRepository,
        FilterGroup $filterGroup,
        FilterBuilder $filterBuilder,
        Status $productStatus,
        Simple $simple_product,
        SearchCriteriaInterface $criteria,
        SearchCriteriaBuilder $searchCriteriaBuilder
    ) {
        $this->productRepository = $productRepository;
        $this->filterGroup = $filterGroup;
        $this->filterBuilder = $filterBuilder;
        $this->productStatus = $productStatus;
        $this->_simple_product = $simple_product;
        $this->criteria = $criteria;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
    }

    public function process()
    {
        parent::process();
        $this->getChilds();
        $this->getPriceType();
        return $this->_item;
    }

    /**
     * @return array
     */
    public function getBundleOptions()
    {
        return $this->bundle_options;
    }

    protected function processChilds()
    {

        /**
         * @var Magento\Catalog\Api\Data\ProductExtension
         */
        $extension_attributes =  $this->_product->getData("extension_attributes");
        $this->bundle_options = [];

        foreach ($extension_attributes->getBundleProductOptions() as $opt) {
            $product_links = $opt->getData('product_links');

            $option =new \stdClass();

            if (is_array($product_links) && count($product_links)) {
                $option->id = (int) $opt->getData("option_id");
                $option->title = $opt->getData("title");
                $option->required = (int)$opt->getData("required");
                $option->type = $opt->getData("type");
                $option->position = (int)$opt->getData("position");
                $option->childs = [];

                foreach ($product_links as $product_link) {
                    $child = new \stdClass();
                    $child->id = (int) $product_link->getData("option_id");
                    $child->product_id = (int)$product_link->getData('entity_id');
                    $child->qty = (int)$product_link->getQty();
                    $child->is_default = (int)$product_link->getIsDefault();
                    $child->can_change_quantity = (int)$product_link->getData('can_change_quantity');
                    $option->childs[] = $child;
                    $this->child_products[] = $child->product_id;
                    unset($child);
                }
            }

            $this->bundle_options[] = $option;
            unset($option);
        }

        return $this->bundle_options;
    }

    protected function getChilds()
    {
        $this->_item->bundle = $this->processChilds();
    }

    /**
     * @return array
     */
    public function getChildProducts()
    {
        return $this->child_products;
    }


    private function getPriceType()
    {
        $priceType = $this->_product->getPriceType() == 0 ? 'dynamic' : 'fixed';
        return $this->_item->bundle_price_type = $priceType;
    }
}
